/*! For license information please see 728.1b5e58477fad37fa95c9.js.LICENSE.txt */
"use strict";(self.webpackChunkgrader_labextension=self.webpackChunkgrader_labextension||[]).push([[728,122,906],{22122:(e,r,t)=>{function n(){return n=Object.assign?Object.assign.bind():function(e){for(var r=1;r<arguments.length;r++){var t=arguments[r];for(var n in t)Object.prototype.hasOwnProperty.call(t,n)&&(e[n]=t[n])}return e},n.apply(this,arguments)}t.d(r,{Z:()=>n})},19756:(e,r,t)=>{function n(e,r){if(null==e)return{};var t,n,o={},c=Object.keys(e);for(n=0;n<c.length;n++)t=c[n],r.indexOf(t)>=0||(o[t]=e[t]);return o}t.d(r,{Z:()=>n})},37078:(e,r,t)=>{t.d(r,{Z:()=>o});const n=e=>e,o=(()=>{let e=n;return{configure(r){e=r},generate:r=>e(r),reset(){e=n}}})()},14142:(e,r,t)=>{t.d(r,{Z:()=>o});var n=t(71387);function o(e){if("string"!=typeof e)throw new Error((0,n.Z)(7));return e.charAt(0).toUpperCase()+e.slice(1)}},94780:(e,r,t)=>{function n(e,r,t=void 0){const n={};return Object.keys(e).forEach((o=>{n[o]=e[o].reduce(((e,n)=>{if(n){const o=r(n);""!==o&&e.push(o),t&&t[n]&&e.push(t[n])}return e}),[]).join(" ")})),n}t.d(r,{Z:()=>n})},59766:(e,r,t)=>{t.d(r,{P:()=>o,Z:()=>s});var n=t(22122);function o(e){return null!==e&&"object"==typeof e&&e.constructor===Object}function c(e){if(!o(e))return e;const r={};return Object.keys(e).forEach((t=>{r[t]=c(e[t])})),r}function s(e,r,t={clone:!0}){const i=t.clone?(0,n.Z)({},e):e;return o(e)&&o(r)&&Object.keys(r).forEach((n=>{"__proto__"!==n&&(o(r[n])&&n in e&&o(e[n])?i[n]=s(e[n],r[n],t):t.clone?i[n]=o(r[n])?c(r[n]):r[n]:i[n]=r[n])})),i}},71387:(e,r,t)=>{function n(e){let r="https://mui.com/production-error/?code="+e;for(let e=1;e<arguments.length;e+=1)r+="&args[]="+encodeURIComponent(arguments[e]);return"Minified MUI error #"+e+"; visit "+r+" for the full message."}t.d(r,{Z:()=>n})},34867:(e,r,t)=>{t.d(r,{Z:()=>c});var n=t(37078);const o={active:"active",checked:"checked",completed:"completed",disabled:"disabled",readOnly:"readOnly",error:"error",expanded:"expanded",focused:"focused",focusVisible:"focusVisible",required:"required",selected:"selected"};function c(e,r,t="Mui"){const c=o[r];return c?`${t}-${c}`:`${n.Z.generate(e)}-${r}`}},1588:(e,r,t)=>{t.d(r,{Z:()=>o});var n=t(34867);function o(e,r,t="Mui"){const o={};return r.forEach((r=>{o[r]=(0,n.Z)(e,r,t)})),o}},18719:(e,r,t)=>{t.d(r,{Z:()=>o});var n=t(66029);function o(e,r){return n.isValidElement(e)&&-1!==r.indexOf(e.type.muiName)}},47925:(e,r,t)=>{t.d(r,{Z:()=>o});var n=t(22122);function o(e,r){const t=(0,n.Z)({},r);return Object.keys(e).forEach((c=>{if(c.toString().match(/^(components|slots)$/))t[c]=(0,n.Z)({},e[c],t[c]);else if(c.toString().match(/^(componentsProps|slotProps)$/)){const s=e[c]||{},i=r[c];t[c]={},i&&Object.keys(i)?s&&Object.keys(s)?(t[c]=(0,n.Z)({},i),Object.keys(s).forEach((e=>{t[c][e]=o(s[e],i[e])}))):t[c]=i:t[c]=s}else void 0===t[c]&&(t[c]=e[c])})),t}},1410:(e,r,t)=>{function n(e){var r,t,o="";if("string"==typeof e||"number"==typeof e)o+=e;else if("object"==typeof e)if(Array.isArray(e))for(r=0;r<e.length;r++)e[r]&&(t=n(e[r]))&&(o&&(o+=" "),o+=t);else for(r in e)e[r]&&(o&&(o+=" "),o+=r);return o}t.d(r,{Z:()=>o});const o=function(){for(var e,r,t=0,o="";t<arguments.length;)(e=arguments[t++])&&(r=n(e))&&(o&&(o+=" "),o+=r);return o}},75251:(e,r,t)=>{var n=t(66029),o=Symbol.for("react.element"),c=Symbol.for("react.fragment"),s=Object.prototype.hasOwnProperty,i=n.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,f={key:!0,ref:!0,__self:!0,__source:!0};function u(e,r,t){var n,c={},u=null,a=null;for(n in void 0!==t&&(u=""+t),void 0!==r.key&&(u=""+r.key),void 0!==r.ref&&(a=r.ref),r)s.call(r,n)&&!f.hasOwnProperty(n)&&(c[n]=r[n]);if(e&&e.defaultProps)for(n in r=e.defaultProps)void 0===c[n]&&(c[n]=r[n]);return{$$typeof:o,type:e,key:u,ref:a,props:c,_owner:i.current}}r.Fragment=c,r.jsx=u,r.jsxs=u},85893:(e,r,t)=>{e.exports=t(75251)}}]);